/*  MegaPad
 *  Copyright (C) 2007 NHerv
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <string.h>
#include <stdarg.h>
#include <sdl/sdl.h>
#include <sdl/sdl_joystick.h>

#include "PAD.h"

#pragma comment(lib, "sdl.lib")
#pragma comment(lib, "sdlmain.lib")

#ifdef _DEBUG
char *libraryName="MegaPad Plugin (debug)";
#else
char *libraryName="MegaPad Plugin";
#endif

Point WinCenter, WinCenterOnScr;
int Logging;
int axisSense;
int pressure[2];
int btnPressed[2][24];

int Byte, Command, Length, Locked[2], ConfMode[2];
char *keyName;
int padopen;
int timer;
int padInit=0;
HWND GShwnd;
WNDPROC GSwndProc;
LRESULT WINAPI PadWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
void SysMessage(char *fmt, ...);
void LoadConfig(), SaveConfig();
s32 Open(void *pDsp);
u8 StartPoll(int pad);
u8 Poll(u8 value);
void Close();
void ShutDown();
void Configure();
void About();
s32 Test();

FILE *padLog=NULL;

u8 *Buffer;
u8 PadMode[2];
int DualShock2;
int sdlResult;

u16 ButtonStat[2]={0xffff, 0xffff};
u8 lX[2]={0x80, 0x80};
u8 lY[2]={0x80, 0x80};
u8 rX[2]={0x80, 0x80};
u8 rY[2]={0x80, 0x80};

u8 Cmd40[2][8];
u8 Cmd41[2][8];
u8 Cmd42[2][20];
u8 Cmd43[2][8];
u8 Cmd44[2][8];
u8 Cmd45[2][8];
u8 Cmd46[2][8];
u8 Cmd47[2][8];
u8 Cmd4C[2][8];
u8 Cmd4D[2][8];
u8 Cmd4F[2][8];

u8 PadID[2];

SDL_Joystick *joy[5][4];
PadConf Pad[2][1];
keyEvent KeyEvent;

void InitPadConf();

void LOG(char *fmt, ...){
	va_list list;
    if(!Logging||padLog==NULL)return;
	va_start(list, fmt);
	vfprintf(padLog, fmt, list);
	va_end(list);
}

s32 CALLBACK PADinit(u32 flags){
	int i;
	if(padLog==NULL){
		padLog=fopen("logs/megapad_log.txt", "w");
		if(padLog==NULL)CreateFile("logs/megapad_log.txt", GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, 0, NULL);
		setvbuf(padLog, NULL, _IONBF, 0);
	}
	LOG("Init\n");
	//Init SDL.
	sdlResult=SDL_Init(SDL_INIT_JOYSTICK);
	//Open each available pad.
	//The plugin supports up to 5 pads.
	if(!SDL_NumJoysticks())
		SysMessage("No joystick available : error %i", SDL_GetError());
	for (i=1; i<=SDL_NumJoysticks(); i++)
	{
		if(i<=5)joy[i-1][4]=SDL_JoystickOpen(i-1);
		if(!SDL_JoystickOpened(i-1))SysMessage("Error %i opening the joystick", GetLastError());
	}
	InitPadConf();
	LoadConfig();
	padInit=1;
	return 0;
}

void CALLBACK PADshutdown(){
	ShutDown();
}

s32 CALLBACK PADopen(void *pDsp){
	return Open(pDsp);
}

void CALLBACK PADclose(){
	Close();
}

u32 CALLBACK PADquery(){
	return 3;
}

u8 CALLBACK PADstartPoll(int pad){
	return StartPoll(pad);
}

void PADsetMode(int pad, int mode){
	PadMode[pad]=mode;
	if(!mode)
	{
		PadID[pad]=0x41;
	}else{
		if(!DualShock2)PadID[pad]=0x73;
		else PadID[pad]=0x79;
	}
}

void SetButtonState(int pad, int index, int pressed, int btnpress){
	pressure[pad]=btnpress;
	if(pressed){
		switch(index){
			case 0: //X
				ButtonStat[pad]&=~(1<<6);
				break;
			case 1: //Square
				ButtonStat[pad]&=~(1<<7);
				break;
			case 2: //Circle
				ButtonStat[pad]&=~(1<<5);
				break;
			case 3: //Triangle
				ButtonStat[pad]&=~(1<<4);
				break;
			case 4: //Up
				ButtonStat[pad]&=~(1<<12);
				break;
			case 5: //Right
				ButtonStat[pad]&=~(1<<13);
				break;
			case 6: //Left
				ButtonStat[pad]&=~(1<<15);
				break;
			case 7: //Down
				ButtonStat[pad]&=~(1<<14);
				break;
			case 8: //Select
				ButtonStat[pad]&=~(1<<8);
				break;
			case 9: //Start
				ButtonStat[pad]&=~(1<<11);
				break;
			case 10: //R1
				ButtonStat[pad]&=~(1<<3);
				break;
			case 11: //L1
				ButtonStat[pad]&=~(1<<2);
				break;
			case 12: //R2
				ButtonStat[pad]&=~(1<<1);
				break;
			case 13: //L2
				ButtonStat[pad]&=~(1<<0);
				break;
			case 14: //R3
				ButtonStat[pad]&=~(1<<10);
				break;
			case 15: //L3
				ButtonStat[pad]&=~(1<<9);
				break;
			case 16: //Right axis - Up
				rY[pad]=128-(btnpress/2);
				break;
			case 17: //Right axis - Right
				rX[pad]=128+(btnpress/2);
				break;
			case 18: //Right axis - Left
				rX[pad]=128-(btnpress/2);
				break;
			case 19: //Right axis - Down
				rY[pad]=128+(btnpress/2);
				break;
			case 20: //Left axis - Up
				lY[pad]=128-(btnpress/2);
				break;
			case 21: //Left axis - Right
				lX[pad]=128+(btnpress/2);
				break;
			case 22: //Left axis - Left
				lX[pad]=128-(btnpress/2);
				break;
			case 23: //Left axis - Down
				lY[pad]=128+(btnpress/2);
				break;
			case 24: //Analog switching
				if(!Locked[pad])PADsetMode(pad, !PadMode[pad]);
				break;
		}
		return;
	}else{
		switch(index){
			case 0: //X
				ButtonStat[pad]|=(1<<6);
				break;
			case 1: //Square
				ButtonStat[pad]|=(1<<7);
				break;
			case 2: //Circle
				ButtonStat[pad]|=(1<<5);
				break;
			case 3: //Triangle
				ButtonStat[pad]|=(1<<4);
				break;
			case 4: //Up
				ButtonStat[pad]|=(1<<12);
				break;
			case 5: //Right
				ButtonStat[pad]|=(1<<13);
				break;
			case 6: //Left
				ButtonStat[pad]|=(1<<15);
				break;
			case 7: //Down
				ButtonStat[pad]|=(1<<14);
				break;
			case 8: //Select
				ButtonStat[pad]|=(1<<8);
				break;
			case 9: //Start
				ButtonStat[pad]|=(1<<11);
				break;
			case 10: //R1
				ButtonStat[pad]|=(1<<3);
				break;
			case 11: //L1
				ButtonStat[pad]|=(1<<2);
				break;
			case 12: //R2
				ButtonStat[pad]|=(1<<1);
				break;
			case 13: //L2
				ButtonStat[pad]|=(1<<0);
				break;
			case 14: //R3
				ButtonStat[pad]|=(1<<10);
				break;
			case 15: //L3
				ButtonStat[pad]|=(1<<9);
				break;
			case 16: //Right axis - Up
				if(rY[pad]>=0 && rY[pad]<=128)rY[pad]=128;
				break;
			case 17: //Right axis - Right
				if(rX[pad]<=255 && rX[pad]>=128)rX[pad]=128;
				break;
			case 18: //Right axis - Left
				if(rX[pad]>=0 && rX[pad]<=128)rX[pad]=128;
				break;
			case 19: //Right axis - Down
				if(rY[pad]<=255 && rY[pad]>=128)rY[pad]=128;
				break;
			case 20: //Left axis - Up
				if(lY[pad]>=0 && lY[pad]<=128)lY[pad]=128;
				break;
			case 21: //Left axis - Right
				if(lX[pad]<=255 && lX[pad]>=128)lX[pad]=128;
				break;
			case 22: //Left axis - Left
				if(lX[pad]>=0 && lX[pad]<=128)lX[pad]=128;
				break;
			case 23: //Left axis - Down
				if(lY[pad]<=255 && lY[pad]>=128)lY[pad]=128;
				break;
		}
	}
}

void SetButtonStateJA(int pad, int index, int axisval, int dir)
{
	int pressed=0, pressure=0;
	if(axisval<=-axisSense || axisval>=axisSense)pressed=1;
	//Pressure-sensitive buttons feature
	if(axisval<=-axisSense && dir==1)pressure=-(axisval/128);
	if(axisval>=axisSense && dir==2)pressure=(axisval/128);
	//Set value
	if(axisval<=-axisSense && dir==1 || axisval>=axisSense && dir==2)
		SetButtonState(pad, index, pressed, pressure);
	else SetButtonState(pad, index, 0, 0);
}

u8 CALLBACK PADpoll(u8 value){
	u8 ret;
	ret=Poll(value);
	LOG("PADpoll : command 0x%x, byte %i, returned 0x%x\n", value, Byte, ret);
	return ret;
}

keyEvent* CALLBACK PADkeyEvent() {
	if(KeyEvent.event!=0){
		return &KeyEvent;
	}
	return NULL;
}

void CALLBACK PADconfigure(){
	Configure();
}

void CALLBACK PADabout(){
	About();
}

s32 CALLBACK PADtest(){
	return Test();
}


